#!/usr/bin/perl
use strict ;
use warnings ;
use autodie ;


use Test::More tests => 47 ;

use Devel::Cover ;
use Data::Dumper ;

use Stanford::Orange::Util qw/ trim / ;

use Stanford::Orange::LogTrend qw/
  find_run_in_files
  find_run_in_files_gzipped
  format_trend_results
  list_files_in_order
/ ;

my $results_aref ;
my (@files, @results, @lines) ;

# Scenario: uncompressed files

# Case 1. one log file: Should return an array with one entry for each
# line of the file

@files = qw|
  ./t/file0.txt
| ;

$results_aref = find_run_in_files(\@files) ;
ok ($results_aref, 'got some result') ;
@results = @{$results_aref} ;

ok (scalar(@results) == 2, 'should be 2 elements of the array') ;
ok (!defined($results[0]), 'first element of array should be undefined') ;

@lines = @{ $results[1] } ;
ok (scalar(@lines) == 6,   'should be 6 lines that occur once') ;

# Case 2. two log files (both files exactly the same). Results should be
# almost the same as Case 1, except the resulting array has all the
# results in the third position.

@files = qw|
  ./t/file0.txt
  ./t/file0.txt
| ;

$results_aref = find_run_in_files(\@files) ;
@results = @{$results_aref} ;
ok (scalar(@results) == 3, 'should be 3 elements of the array' ) ;
ok (!defined($results[0]) && !defined($results[1]),
                           'first two elements should be undefined') ;

@lines = @{$results[2]} ;
ok (scalar(@lines) == 6,   'should be 6 lines that occur 2 times') ;

# Case 3. two log files, both files the same except second file has lines
# in a different order.

@files = qw|
  ./t/file0.txt
  ./t/file0_scrambled.txt
| ;

$results_aref = find_run_in_files(\@files) ;
@results = @{$results_aref} ;
ok (scalar(@results) == 3) ;
ok (!defined($results[0])) ;
ok (!defined($results[1])) ;
@lines = @{ $results[2] } ;
ok (scalar(@lines) == 6) ;
ok ($lines[0] = 'A' &&
    $lines[1] = 'B' &&
    $lines[2] = 'C' &&
    $lines[3] = 'D' &&
    $lines[4] = 'E' &&
    $lines[5] = 'F') ;


# Case 4. two log files, second file a subset of the first.

@files = qw|
  ./t/file0.txt
  ./t/file1.txt
| ;

$results_aref = find_run_in_files(\@files) ;
@results = @{$results_aref} ;
ok (scalar(@results) == 3) ;
ok (!defined($results[0])) ;

@lines = @{ $results[1] } ;
ok (scalar(@lines) == 1) ;

@lines = @{ $results[2] } ;
ok (scalar(@lines) == 5) ;
ok ($lines[0] = 'A' &&
    $lines[1] = 'B' &&
    $lines[2] = 'D' &&
    $lines[3] = 'E' &&
    $lines[4] = 'F') ;


# Case 5. two log files, second file NOT a subset of the first.
#
# file0:
# A
# B
# C
# D
# E
# F
#
# file2:
# Q
# B
# C
# D
# E
# F
# A
# Z


@files = qw|
  ./t/file0.txt
  ./t/file2.txt
| ;

$results_aref = find_run_in_files(\@files) ;

@results = @{$results_aref} ;
ok (scalar(@results) == 3) ;
ok (!defined($results[0])) ;
ok (!defined($results[1])) ;

@lines = @{ $results[2] } ;
ok (scalar(@lines) == 6) ;
ok ($lines[0] = 'A' &&
    $lines[1] = 'B' &&
    $lines[2] = 'C' &&
    $lines[3] = 'D' &&
    $lines[4] = 'E' &&
    $lines[5] = 'F') ;



# Case 6. three log files.

@files = qw|
  ./t/file0.txt
  ./t/file1.txt
  ./t/file2.txt
| ;

$results_aref = find_run_in_files(\@files) ;
@results = @{$results_aref} ;
ok (scalar(@results) == 4) ;

ok (!defined($results[0])) ;
ok (scalar(@{ $results[1] }) == 1) ;
ok (!defined($results[2])) ;
ok (scalar(@{ $results[3] }) == 5) ;


@lines = @{ $results[1] } ;
ok (scalar(@lines) == 1) ;

@lines = @{ $results[3] } ;
ok (scalar(@lines) == 5) ;
ok ($lines[0] = 'A' &&
    $lines[1] = 'B' &&
    $lines[2] = 'D' &&
    $lines[3] = 'E' &&
    $lines[4] = 'F') ;


# Now do all three files compressed!
@files = qw|
  ./t/file0.txt.gz
  ./t/file1.txt.gz
  ./t/file2.txt.gz
| ;

$results_aref = find_run_in_files_gzipped(\@files) ;
@results = @{$results_aref} ;
ok (scalar(@results) == 4) ;

ok (!defined($results[0])) ;
ok (scalar(@{ $results[1] }) == 1) ;
ok (!defined($results[2])) ;
ok (scalar(@{ $results[3] }) == 5) ;


@lines = @{ $results[1] } ;
ok (scalar(@lines) == 1) ;

@lines = @{ $results[3] } ;
ok (scalar(@lines) == 5) ;
ok ($lines[0] = 'A' &&
    $lines[1] = 'B' &&
    $lines[2] = 'D' &&
    $lines[3] = 'E' &&
    $lines[4] = 'F') ;


my $formatted = format_trend_results($results_aref) ;
my $answer = <<'EOS';
Appears in 3 consecutive days
--------------------
A
B
D
E
F

Appears in 1 consecutive day
--------------------
C
EOS

ok (trim($formatted) eq trim($answer)) ;


# Test list_files_in_order

# Set the file modified times.
set_file_times() ;

my @files_sorted ;

@files_sorted = list_files_in_order('./t/', '^file..txt$') ;
ok (scalar(@files_sorted) == 3) ;
ok ($files_sorted[0] =~ m{file2.txt$}xsm) ;
ok ($files_sorted[1] =~ m{file1.txt$}xsm) ;
ok ($files_sorted[2] =~ m{file0.txt$}xsm) ;

@files_sorted = list_files_in_order('./t/', '^sorted\d\.txt$') ;
ok (scalar(@files_sorted) == 3) ;
ok ($files_sorted[0] =~ m{sorted1.txt$}xsm) ;
ok ($files_sorted[1] =~ m{sorted2.txt$}xsm) ;
ok ($files_sorted[2] =~ m{sorted0.txt$}xsm) ;


# Need to set the file times on these files so they have the correct
# order.
sub set_file_times {
    my $time = time() ;

    utime $time,      $time,      './t/file0.txt' ;
    utime $time + 30, $time + 30, './t/file1.txt' ;
    utime $time + 60, $time + 60, './t/file2.txt' ;

    utime $time,      $time,      './t/sorted0.txt' ;
    utime $time + 30, $time + 30, './t/sorted2.txt' ;
    utime $time + 60, $time + 60, './t/sorted1.txt' ;

    return ;
}
